import os
from tkinter import Tk
from tkinter import filedialog
from kskit import java
import getpass

def get_home():
  if os.environ.get('DP_HOME') == None :
    print("Please select the deep.piste home folder")
    os.environ['DP_HOME'] = filedialog.askdirectory()
    print(f"you have set  {os.environ.get('DP_HOME')} as epi home")
  return os.environ.get('DP_HOME')

def get_password(name, message = None):
  if os.environ.get(f'DP_PWD_{name.upper()}') == None :
    return getpass.getpass(prompt=f"{message}:", stream=None)
  return os.environ.get(f'DP_PWD_{name.upper()}')

def prepare_sparkly(repo):
  home = get_home()
  jars_path = os.path.join(home, "libs/jars")
  os.makedirs(jars_path, exist_ok=True)
  
  package_dir = os.path.abspath(os.path.dirname(__file__))
  sparkly_paths = os.path.join(package_dir, "data", "sparkly-maven-paths.url")
  
  java.build_classpath(repo, sparkly_paths, jars_path)


def sparkly_cp(source, dest):
  """
    Moving files from sparkly supported storage systems
  """ 
  java.java_job(main_class = "fr.epiconcept.sparkly.Command", class_path = f"{os.path.join(get_home(), 'libs', 'jars')}" , memory = "1g", args = ["storage", "cp", "from", source, "to", dest, "force", "true"])



from pydicom.dataset import Dataset
from pynetdicom import (
    AE, build_role, evt,
    StoragePresentationContexts,
    QueryRetrievePresentationContexts,
    debug_logger
)

from pynetdicom.sop_class import (
    PatientRootQueryRetrieveInformationModelGet,
    StudyRootQueryRetrieveInformationModelGet,
    PatientStudyOnlyQueryRetrieveInformationModelGet,
    PlannedImagingAgentAdministrationSRStorage,
    PerformedImagingAgestAdministrationSRStorage,
    EncapsulatedSTLStorage,
)

debug_logger()
storage_dest = ""
# Implement the handler for evt.EVT_C_STORE
def handle_store(event):
    """Handle a C-STORE request event."""
    ds = event.dataset
    ds.file_meta = event.file_meta

    # Save the dataset using the SOP Instance UID as the filename
    ds.save_as(f"{storage_dest}/{ds.SOPInstanceUID}", write_like_original=False)

    # Return a 'Success' status
    return 0x0000

def get_dicom(seriesInstanceUID, dest, server = "127.0.0.1", port = 11112, title = "ANY"):
  handlers = [(evt.EVT_C_STORE, handle_store)]
  global storage_dest 
  storage_dest = dest
  # Initialise the Application Entity
  ae = AE()
  
  
  _exclusion = [
      PlannedImagingAgentAdministrationSRStorage,
      PerformedImagingAgestAdministrationSRStorage,
      EncapsulatedSTLStorage,
  ]
  store_contexts = [
      cx for cx in StoragePresentationContexts
      if cx.abstract_syntax not in _exclusion
  ]
  # Extended Negotiation - SCP/SCU Role Selection
  ext_neg = []
  ae.add_requested_context(PatientRootQueryRetrieveInformationModelGet)
  ae.add_requested_context(StudyRootQueryRetrieveInformationModelGet)
  ae.add_requested_context(PatientStudyOnlyQueryRetrieveInformationModelGet)
  for cx in store_contexts:
      ae.add_requested_context(cx.abstract_syntax)
      # Add SCP/SCU Role Selection Negotiation to the extended negotiation
      # We want to act as a Storage SCP
      ext_neg.append(build_role(cx.abstract_syntax, scp_role=True))
  
  
  
  # Create our Identifier (query) dataset
  # We need to supply a Unique Key Attribute for each level above the
  #   Query/Retrieve level
  ds = Dataset()
  ds.QueryRetrieveLevel = 'SERIES'
  # Unique key for PATIENT level
  #ds.PatientID = '1234567'
  # Unique key for STUDY level
  #ds.StudyInstanceUID = '1.2.3'
  # Unique key for SERIES level
  ds.SeriesInstanceUID = seriesInstanceUID
  
  # Associate with peer AE at IP 127.0.0.1 and port 11112
  assoc = ae.associate(server, port, ae_title=title, ext_neg=ext_neg, evt_handlers=handlers)
  
  if assoc.is_established:
      # Use the C-GET service to send the identifier
      responses = assoc.send_c_get(ds, PatientRootQueryRetrieveInformationModelGet)
      for (status, identifier) in responses:
          if status:
              print('C-GET query status: 0x{0:04x}'.format(status.Status))
          else:
              print('Connection timed out, was aborted or received invalid response')
  
      # Release the association
      assoc.release()
  else:
      print('Association rejected, aborted or never connected')
  
get_dicom(seriesInstanceUID='1.3.6.1.4.1.9590.100.1.2.25530538411839768118282182472795553760', dest = "/tmp/pynetdicom", server = "127.0.0.1", port = 11112, title = b'DCM4CHEE')




from pydicom import dcmread
from pydicom.data import get_testdata_file
from matplotlib import pyplot
from difflib import Differ

infile = get_testdata_file("MR_small.dcm")

infiles = [
    "/home/fod/datapub/cbis-ddsm/Calc-Test_P_00246_RIGHT_MLO_1/1.3.6.1.4.1.9590.100.1.2.12392078612698738504735382232322772600"
   ,"/home/fod/datapub/cbis-ddsm/CBIS-DDSM/Calc-Test_P_00246_RIGHT_MLO/08-29-2017-DDSM-55726/1.000000-full mammogram images-53760/1-1.dcm"
 ]


def write_dicom(infiles):
  i = 0
  for infile in infiles:
    outfile = f"/home/fod/deleteme/dicom_{i}.png"
    ds = dcmread(infile)
    pixels = ds.pixel_array
    pyplot.imsave(outfile, pixels, cmap=pyplot.cm.bone)
    print(f"file {outfile} written")
    i = i +1

def read_dicom(infiles):
  for infile in infiles:
    ds = dcmread(infile)
    yield((infile, ds) )

def compare_dicom(infiles):
  lines = list([str(df).splitlines(keepends = True) for (f, df) in read_dicom(infiles)])
  difs = Differ().compare(lines[0], lines[1])
  return filter(lambda l: l[0:2] != "  ", difs)





import sys
import argparse
from tkinter import Tk
from tkinter.filedialog import askopenfilename
from .p02_initial_extraction import *
from . import utils

def main(a):
  # Base argument parser
  parser = argparse.ArgumentParser()
  subs = parser.add_subparsers()
  
  # extract command
  extract_parser = subs.add_parser("extract", help = "Invoke initial extractions commands") 
  extract_subs = extract_parser.add_subparsers()

  # extract neoscope command
  neoextract_parser = extract_subs.add_parser("neoscope", help = "Invoke neoscope extractions commands") 
  neoextract_subs = neoextract_parser.add_subparsers()

  # -- get neoscope key to clipboard
  neokey2clipboard_parser = neoextract_subs.add_parser("readkey", aliases=['key'], help = "Copy the neoscope extraction key from qrcode to clipboard")
  neokey2clipboard_parser.set_defaults(func = do_neokey2clipboard)

  # -- encrypt neoscope extractions
  encrypt_neoextract_parser = neoextract_subs.add_parser("encrypt", help = "Encrypt neoscope extractions")
  encrypt_neoextract_parser.add_argument("-e", "--extractions", required=False, help="Path to a file containing the extraction files to be encryped. If not provided it will be prompted")
  encrypt_neoextract_parser.set_defaults(func = do_encrypt_neoscope_extractions)

  # -- upload encrypted files to epifiles
  neoextract2epifiles_parser = neoextract_subs.add_parser("push2epifiles", aliases=["push"], help = "Push neoscope extractions to epifiles")
  neoextract2epifiles_parser.add_argument(
    "-e", 
    "--epifiles-host", 
    required=False, 
    help="epifiles base host, default = epifiles.voozanoo.net", 
    default = "epifiles.voozanoo.net"
  )
  neoextract2epifiles_parser.add_argument(
    "-r", 
    "--maven-repo", 
    required=False, 
    help="maven repository to use to download sparkly and dependencies, default = https://repo1.maven.org/maven2", 
    default = "https://repo1.maven.org/maven2"
  )
  neoextract2epifiles_parser.add_argument("-u", "--epi-user", required=True, help="login for connecting to epifiles")
  neoextract2epifiles_parser.set_defaults(func = do_neoextract2epifiles)

  # -- download encrypted files from epifiles
  epifiles2neoextract_parser = neoextract_subs.add_parser("pull2bigdata", aliases=["pull"], help = "Pull neoscope extractions from epifiles to big data infrastructure")
  epifiles2neoextract_parser.add_argument(
    "-e", 
    "--epifiles-host", 
    required=False, 
    help="epifiles base host, default = epifiles.voozanoo.net", 
    default = "epifiles.voozanoo.net"
  )
  epifiles2neoextract_parser.add_argument(
    "-r", 
    "--maven-repo", 
    required=False, 
    help="maven repository to use to download sparkly and dependencies, default = https://repo1.maven.org/maven2", 
    default = "https://repo1.maven.org/maven2"
  )
  epifiles2neoextract_parser.add_argument("-u", "--epi-user", required=True, help="login for connecting to epifiles")
  epifiles2neoextract_parser.set_defaults(func = do_epifiles2bigdata)
  
  # -- decrypt neoscope extractions
  decrypt_neoextract_parser = neoextract_subs.add_parser("decrypt", help = "Decrypt neoscope extractions")
  decrypt_neoextract_parser.set_defaults(func = do_decrypt_neoscope_extractions)
   
  # calling handlers
  args = parser.parse_args()
  try:
    func = args.func
  except AttributeError:
    parser.error("too few arguments add -h to see your options")
  
  args.func(args)
  

# handlers
def do_neokey2clipboard(args):
  p02_002_neoscope_key_to_clipboard()

def do_encrypt_neoscope_extractions(args):
  get_home()
  if args.extractions == None:
    print("Please chose the file to crypt")
    Tk().withdraw()
    filename = askopenfilename()
  else:
    filename = args.extractions
  p02_003_encrypt_neoscope_extractions(filename)   

def do_neoextract2epifiles(args):
  utils.prepare_sparkly(repo = args.maven_repo)
  p02_004_send_neoscope_extractions_to_epifiles(
    epifiles = args.epifiles_host, 
    login=args.epi_user, 
    password=utils.get_password(f"epifiles", f"Password for {args.epi_user}")
  )
def do_epifiles2bigdata(args):
  utils.prepare_sparkly(repo = args.maven_repo)
  p02_005_get_neoscope_extractions_from_epifiles(
    epifiles = args.epifiles_host, 
    login=args.epi_user, 
    password=utils.get_password(f"epifiles", f"Password for {args.epi_user}")
  )
def do_decrypt_neoscope_extractions(args):
  p02_006_decrypt_neoscope_extractions()   

if __name__ == "__main__":
  main(sys.argv[1])





import kskit
import os
import base64
import clipboard
import urllib.parse
from tkinter import Tk
from tkinter import filedialog
from .utils import get_home, sparkly_cp


def p02_001_generate_neoscope_key():
  """Generate a 256bit random QR code to be used as an AES encryption simmetric key"""
  dp_home = get_home()
  neo_key_path = os.path.join(dp_home, "neo_key.png")
  kskit.generate_qr_key(neo_key_path, 32) #32 bytes = 256bits
  print(f"Neoscope extractios key produced on '{neo_key_path}'. Please print too copies of this file, send one to Neoscope operator end delete this file")
  
def p02_002_neoscope_key_to_clipboard():   
  """Use user webcam to copy the neoscope encryption base 64 key to the clipboard"""
  b64key = kskit.read_webcam_key(auto_close = True, camera_index = 0)
  clipboard.copy(b64key)
  print(f"the key has been copied to the clipboard, put it to the server clipboard and call p02_003_encrypt_neoscope_extractions on it")

def p02_003_encrypt_neoscope_extractions(source):   
  """Encrypts the provided file to the dp_home directory using a clipboard pasted base 64 key"""
  dest = os.path.join(get_home(), "extraction_neoscope.aes")
  b64key = clipboard.paste()
  kskit.encrypt(source, dest, b64key)
  #cleaning clipboard
  clipboard.copy("")
  print(f"file has been encrypted and stored on {dest} please proceed delete {source}, send to epiconcept via Epifiles and then remove {dest} too")

def p02_004_send_neoscope_extractions_to_epifiles(epifiles, login, password):
  """Sends the encrypted neoscope extractions to epifiles"""
  source = os.path.join(get_home(), "extraction_neoscope.aes")
  dest = f"epi://{urllib.parse.quote_plus(login)}:{urllib.parse.quote_plus(password)}@{epifiles}/extraction_neoscope.aes"
  sparkly_cp(source = source, dest = dest) 

def p02_005_get_neoscope_extractions_from_epifiles(epifiles, login, password):
  """Gets the encrypted neoscope extractions from epifiles"""
  dest = os.path.join(get_home(), "extraction_neoscope.aes")
  source = f"epi://{urllib.parse.quote_plus(login)}:{urllib.parse.quote_plus(password)}@{epifiles}/extraction_neoscope.aes"
  sparkly_cp(source = source, dest = dest) 

def p02_006_decrypt_neoscope_extractions():   
  """Decrypts the provided file to the dp_home directory using a key pasted on the clipboard"""
  crypted = os.path.join(get_home(), "extraction_neoscope.aes")
  orig = os.path.join(get_home(), "extraction_neoscope.zip")
  b64key = clipboard.paste()
  kskit.decrypt(crypted, orig, b64key)
  print(f"file has been decrypted and stored on {orig} please proceed delete {crypted} and run the extractions")





from pydicom import dcmread
from matplotlib import pyplot

def dicom2png(infile, outfile):
  ds = dcmread(infile)
  pixels = ds.pixel_array
  pyplot.imsave(outfile, pixels, cmap=pyplot.cm.bone)
  print(f"file {outfile} written")


dicom2png("/tmp/pynetdicom/1.3.6.1.4.1.9590.100.1.2.12392078612698738504735382232322772600", "/tmp/pynetdicom/1.3.6.1.4.1.9590.100.1.2.12392078612698738504735382232322772600.png")



import sys
import requests 
from requests.auth import HTTPBasicAuth 
import xmltodict
import json
import re
from types import SimpleNamespace
import pandas 
import numpy
    
def read_response(r):
  if r.status_code == 200 : 
    if re.search("xml|html", r.headers["content-type"]) != None :
      resp_dict=xmltodict.parse(r.text)
      return json.loads(json.dumps(resp_dict), object_hook=lambda d: SimpleNamespace(**d))
    elif re.search("json", r.headers["content-type"]) != None :
      return r.json(object_hook=lambda d: SimpleNamespace(**d))
    else :
      raise NotImplementedError(f"Handling content type { r.headers['content-type'] } is not supported @epi")
  else :
    raise ConnectionError(f"Got status {r.status_code}\n {r.text}")

def voo_parse(value, d_type):
  if value == None :
    return None
  elif d_type == "string":
    return str(value)
  elif d_type == "integer":
    return int(value)
  elif d_type == "primary_key":
    return int(value)
  elif d_type == "fkey_dico":
    return int(value)
  elif d_type == "fkey_varset":
    return int(value)
  elif d_type == "fkey_dico_ext":
    return str(value)
  elif d_type == False:
    return str(value)
  elif d_type == "date":
    return numpy.datetime64(value)
  elif d_type == "datetime":
    return numpy.datetime64(value)
  else:
    raise NotImplementedError(f"Parsing {value} as {d_type} is not yet implemented")

def voo_type(columns):
  for (name, column) in columns.items():
    d_type = column.type
    if d_type == "string":
      yield (name, numpy.str)
    elif d_type == "integer":
      yield (name, "Int64")
    elif d_type == "primary_key":
      yield (name, "Int64")
    elif d_type == "fkey_dico":
      yield (name, "Int64")
    elif d_type == "fkey_varset":
      yield (name, "Int64")
    elif d_type == "fkey_dico_ext":
      yield (name, "string")
    elif d_type == False:
      yield (name, "string")
    elif d_type == "date":
      yield (name, numpy.datetime64)
    elif d_type == "datetime":
      yield (name, numpy.datetime64)
    else:
      raise NotImplementedError(f"Parsing type '{d_type}' is not yet implemented")

def generate_rows(rows, columns) : 
  for r in rows:
    yield [voo_parse(getattr(r, field), columns[field].type) for field in list(columns.keys())] 

def get_datasets(voo_url, login, password) :
  r = requests.get(f'{voo_url}/ws/dataset', auth = HTTPBasicAuth(login, password)) 
  # print datasets
  ds = read_response(r)
  for d in ds.root.response.dataset :
    print(f"{d.id}\t{d.resource_id}\t{d.name}")

voo_url = 'https://neoesis.preprod.voozanoo.net/neodemat'
login = 'f.orchard'

if len(sys.argv) == 2 :
  get_datasets(voo_url, login, sys.argv[1])
elif len(sys.argv) == 3 : 
  r = requests.get(f'{voo_url}/ws/dataset/id/{int(sys.argv[2])}/format/json', auth = HTTPBasicAuth(login, sys.argv[1])) 
  ds = read_response(r)
  columns = dict([(n, getattr(ds.metadata.fields,n )) for n in dir(ds.metadata.fields) if re.match("^__", n) == None])
  data = generate_rows(ds.rowdata, columns)
  df = pandas.DataFrame(data = data, columns = columns.keys()).astype(dict(voo_type(columns)))
  print(df)

  #print(ds) 
#id/'+ oItem.id +'/format/json/



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.md') as f:
    readme = f.read()

setup(
    name='deep-piste',
    version='0.0.4',
    description='Evaluating the impact of IA on Breast Cancer Screening Program',
    long_description=readme,
    long_description_content_type='text/markdown',
    author='Francisco Orchard',
    author_email='f.orchard@epiconcept.fr',
    url='https://github.com/Epiconcept-Paris/deep.piste',
    license="GNU General Public License v2.0",
    install_requires=[
      "kskit",
      "clipboard"
    ],
    packages=find_packages(exclude=('tests', 'docs')),
    include_package_data=True
)

